import pytest
from faker import Faker
from orchestrator.db import SubscriptionTable
from orchestrator.services import subscriptions

from gso.products.product_blocks.router import RouterRole, RouterVendor
from gso.products.product_blocks.site import SiteTier


class TestImportEndpoints:
    @pytest.fixture(autouse=True)
    def setup(self, test_client):
        self.faker = Faker()
        self.client = test_client
        self.site_import_endpoint = "/api/imports/sites"
        self.router_import_endpoint = "/api/imports/routers"
        self.site_data = {
            "site_name": self.faker.name(),
            "site_city": self.faker.city(),
            "site_country": self.faker.country(),
            "site_country_code": self.faker.country_code(),
            "site_latitude": float(self.faker.latitude()),
            "site_longitude": float(self.faker.longitude()),
            "site_bgp_community_id": self.faker.pyint(),
            "site_internal_id": self.faker.pyint(),
            "site_tier": SiteTier.TIER1,
            "site_ts_address": self.faker.ipv4(),
            "customer": "Geant",
        }
        self.router_data = {
            "hostname": "127.0.0.1",
            "router_role": RouterRole.PE,
            "router_vendor": RouterVendor.JUNIPER,
            "router_site": self.site_data["site_name"],
            "ts_port": 1234,
            "customer": "Geant",
            "is_ias_connected": True,
            "router_lo_ipv4_address": self.faker.ipv4(),
            "router_lo_ipv6_address": self.faker.ipv6(),
            "router_lo_iso_address": "TestAddress",
        }

    def test_import_site_endpoint(self):
        assert SubscriptionTable.query.all() == []
        # Post data to the endpoint
        response = self.client.post(self.site_import_endpoint, json=self.site_data)
        assert response.status_code == 201
        assert "detail" in response.json()
        assert "pid" in response.json()
        subscription = subscriptions.retrieve_subscription_by_subscription_instance_value(
            resource_type="site_name", value=self.site_data["site_name"]
        )
        assert subscription is not None
        self.site_data.pop("customer")

    def test_import_site_endpoint_with_existing_site(self):
        response = self.client.post(self.site_import_endpoint, json=self.site_data)
        assert SubscriptionTable.query.count() == 1
        assert response.status_code == 201

        response = self.client.post(self.site_import_endpoint, json=self.site_data)
        assert response.status_code == 409
        assert SubscriptionTable.query.count() == 1

    def test_import_site_endpoint_with_invalid_data(self):
        # invalid data, missing site_latitude and invalid site_longitude
        self.site_data.pop("site_latitude")
        self.site_data["site_longitude"] = "invalid"
        assert SubscriptionTable.query.count() == 0
        response = self.client.post(self.site_import_endpoint, json=self.site_data)
        assert response.status_code == 422
        assert SubscriptionTable.query.count() == 0
        response = response.json()
        assert response["detail"][0]["loc"] == ["body", "site_latitude"]
        assert response["detail"][0]["msg"] == "field required"
        assert response["detail"][1]["loc"] == ["body", "site_longitude"]
        assert response["detail"][1]["msg"] == "value is not a valid float"

    def test_import_router_endpoint(self):
        # Create a site first
        response = self.client.post(self.site_import_endpoint, json=self.site_data)
        assert response.status_code == 201
        assert SubscriptionTable.query.count() == 1

        response = self.client.post(self.router_import_endpoint, json=self.router_data)
        assert response.status_code == 201
        assert SubscriptionTable.query.count() == 2

    def test_import_router_endpoint_with_invalid_data(self):
        response = self.client.post(self.site_import_endpoint, json=self.site_data)
        assert response.status_code == 201
        assert SubscriptionTable.query.count() == 1

        # invalid data, missing hostname and invalid router_lo_ipv6_address
        self.router_data.pop("hostname")
        self.router_data["router_lo_ipv6_address"] = "invalid"
        response = self.client.post(self.router_import_endpoint, json=self.router_data)
        assert response.status_code == 422
        assert SubscriptionTable.query.count() == 1
        response = response.json()
        assert response["detail"][0]["loc"] == ["body", "hostname"]
        assert response["detail"][0]["msg"] == "field required"
        assert response["detail"][1]["loc"] == ["body", "router_lo_ipv6_address"]
        assert response["detail"][1]["msg"] == "value is not a valid IPv6 address"
